# Hunter's Most Active Addon
#  
# Install instructions:
#       1. Install Mattie's EventScripts 2.0 plugin:
#           http://mattie.info/cs/
#
#       2. Copy and upload this script to:
#           <gamedir>/addons/eventscripts/most_active/
#
#       3. Add the following line somewhere in autoexec.cfg:
#           es_load most_active
#
################################################################################
#import EventScripts
import es
#import Libraries
import gamethread
import keyvalues
import playerlib
import popuplib
import time
import os
from sqlite3 import dbapi2 as sqlite
#import Psyco Compiler
import psyco
psyco.full()

# Addon Information
info = es.AddonInfo()
info.name = "Most Active"
info.version = "4.0.1"
info.author = "Hunter"
info.url = "http://addons.eventscripts.com/addons/user/289"
info.description = "Tracks the played time of each player on your server"
info.basename = "most_active"

hunter_most_active_ver      = info.version
hunter_most_active_text     = 'Hunters '+info.name+', www.sourceplugins.de, '+info.version+', ES 2.0.0.247+ (Python Version)'

# Rename Database
if os.path.exists(es.getAddonPath('most_active')+'/es_most_active_storage.sqldb'):
    os.rename(es.getAddonPath('most_active')+'/es_most_active_storage.sqldb', es.getAddonPath('most_active')+'/most_active.sqldb')

# Server Variables
most_active_savetime        = es.ServerVar('most_active_savetime', '30', 'How much days should inactive users be stored, before their settings get deleted? ( default = 30 )')
most_active_msg_connect     = es.ServerVar('most_active_msg_connect', '1', 'Should the scriptaddon annouce player time stats on connect? ( 1 = on | 0 = off )')
most_active_msg_disconnect  = es.ServerVar('most_active_msg_disconnect', '1', 'Should the scriptaddon annouce player time stats on disconnect? ( 1 = on | 0 = off )')
most_active_msg_teamchange  = es.ServerVar('most_active_msg_teamchange', '1', 'Should the scriptaddon annouce player time stats on teamchange? ( 1 = on | 0 = off )')
most_active_ignore_bots     = es.ServerVar('most_active_ignore_bots', '0', 'Ignore bots? ( 1 = yes | 0 = no )')
most_active_usertags        = es.ServerVar('most_active_usertags', '0', 'Enable UserTags feature? ( 1 = on | 0 = off )')
most_active_usermodels      = es.ServerVar('most_active_usermodels', '0', 'Enable UserModels feature? ( 1 = on | 0 = off )')

# Global Variables
most_active_players         = {}
most_active_popups          = {}

# KeyValues Object
most_active_kv              = keyvalues.KeyValues(name='most_active')

# SQLite Object
most_active_db              = sqlite.connect(es.getAddonPath('most_active')+'/most_active.sqldb')

# Player Object
class Player(object):
    def __init__(self, userid):
        self.data = {}
        self.temp = {}
        self.userid = int(userid)
        self.player = playerlib.getPlayer(self.userid)
        self.uniqueid = playerlib.uniqueid(self.userid, True)
        self.cursor = most_active_db.cursor()
        self.cursor.execute('SELECT * FROM PlayerList WHERE uniqueid=? LIMIT 1;', (self.uniqueid,))
        if self.cursor.fetchone():
            self.cursor.execute('UPDATE PlayerList SET name_last=?, time_last=? WHERE uniqueid=?;', (self.player.attributes['name'], int(_gettimestamp()), self.uniqueid))
        else:
            self.cursor.execute('REPLACE INTO PlayerList (uniqueid,name,name_last,time_first,time_last,time_total,time_team0,time_team1,time_team2,time_team3) VALUES (?,?,?,?,?,0,0,0,0,0);', (self.uniqueid, self.player.attributes['name'], self.player.attributes['name'], int(_gettimestamp()), int(_gettimestamp())))
        self.temp = {'session_total':self.time_total,'session_team0':self.time_team0,'session_team1':self.time_team1,'session_team2':self.time_team2,'session_team3':self.time_team3}
    def __getattr__(self, name):
        if name in ['name','name_last','time_first','time_last','time_total','time_team0','time_team1','time_team2','time_team3','id']:
            if not name in self.data:
                self.cursor.execute('SELECT %s FROM PlayerList WHERE uniqueid=? LIMIT 1;' % name, (self.uniqueid,))
                self.data[name] = self.cursor.fetchone()[0]
            return self.data[name]
        elif name in self.temp and name.startswith('session_'):
            return self.data[name.replace('session_', 'time_')]-self.temp[name]
        elif name in self.temp:
            return self.temp[name]
    def __setattr__(self, name, value):
        if name in ['name','name_last','time_first','time_last','time_total','time_team0','time_team1','time_team2','time_team3']:
            self.cursor.execute('UPDATE PlayerList SET %s=? WHERE uniqueid=?;' % name, (value, self.uniqueid))
            self.data[name] = value
        else:
            self.__dict__[name] = value

def load():
    public = es.ServerVar('hu_ma', info.version, info.name)
    public.makepublic()

    most_active_kv.load(es.getAddonPath('most_active')+'/most_active.txt')
    most_active_db.cursor().execute('CREATE TABLE IF NOT EXISTS PlayerList (id INTEGER DEFAULT 1 NOT NULL PRIMARY KEY,uniqueid TEXT  UNIQUE NOT NULL,name TEXT  NOT NULL,name_last TEXT  NOT NULL,time_first INTEGER  NOT NULL,time_last INTEGER  NOT NULL,time_total INTEGER  NULL,time_team0 INTEGER  NULL,time_team1 INTEGER  NULL,time_team2 INTEGER  NULL,time_team3 INTEGER  NULL);')
    most_active_db.commit()

    es.regsaycmd('!active', 'most_active/saycmd', 'Most Active Stats')
    es.log(hunter_most_active_text)
    es.msg('#multi', '#green[MostActive] #defaultLoaded')

def unload():
    for popup in most_active_popups:
        most_active_popups[popup].delete()

    most_active_db.commit()
    most_active_db.close()

    es.unregsaycmd('!active')
    es.msg('#multi', '#green[MostActive] #defaultUnloaded')
    
def es_map_start(event_var):
    most_active_db.cursor().execute('CREATE TABLE IF NOT EXISTS PlayerList (id INTEGER DEFAULT 1 NOT NULL PRIMARY KEY,uniqueid TEXT  UNIQUE NOT NULL,name TEXT  NOT NULL,name_last TEXT  NOT NULL,time_first INTEGER  NOT NULL,time_last INTEGER  NOT NULL,time_total INTEGER  NULL,time_team0 INTEGER  NULL,time_team1 INTEGER  NULL,time_team2 INTEGER  NULL,time_team3 INTEGER  NULL);')
    most_active_db.cursor().execute('DELETE FROM PlayerList WHERE time_last<?;', (int(_gettimestamp())-(int(most_active_savetime)*86400),))
    most_active_db.commit()

def es_client_command(event_var):
    if (str(event_var['command']) == '!hunter_most_active_ver') or (str(event_var['command']) == '!hunter_all_ver'):
        es.cexec(int(event_var['userid']), 'echo '+hunter_most_active_text)

def player_activate(event_var):
    gamethread.delayed(30, es.cexec, (int(event_var['userid']), 'echo '+hunter_most_active_text))
    gamethread.delayed(15, es.tell, (int(event_var['userid']), '#multi', '#green[MostActive] #defaultSay \'!active\' for activity stats'))

    player = _player(event_var['userid'])

    if not int(player.time_total) and int(most_active_msg_connect):
        gamethread.delayed(10, es.msg, ('#multi', '#green[MostActive] #lightgreen%s#default will be tracked!' % player.player.attributes['name']))
    elif int(most_active_msg_connect):
        gamethread.delayed(20, es.msg, ('#multi', '#green[MostActive] #lightgreen%s\'s #defaultplaying times: Total %s' % (player.player.attributes['name'], _format(player.time_total))))
    if int(most_active_usertags):
        gamethread.delayed(60, _checkname, (event_var['userid'], True))

def player_disconnect(event_var):
    if int(event_var['userid']) in most_active_players:
        _calculate(event_var['userid'])

        if int(most_active_msg_disconnect):
            es.msg('#multi', '#green[MostActive] #lightgreen%s\'s #defaultplaying times: Total %s - Session %s' % (event_var['es_username'], _format(_gettotaltime(event_var['userid'])), _format(_gettotaltime(event_var['userid'], True))))
        del most_active_players[int(event_var['userid'])]

    most_active_db.commit()

def player_spawn(event_var):
    if not int(event_var['userid']) in most_active_players: return

    if int(most_active_usermodels):
        gamethread.queue(_checkmodel, (event_var['userid']))

def player_death(event_var):
    _calculate(event_var['userid'])

    most_active_db.commit()

def player_team(event_var):
    if int(event_var['disconnect']): return

    _calculate(event_var['userid'], event_var['oldteam'])

    if int(event_var['team']) != int(event_var['oldteam']) and int(event_var['oldteam']) > 1:
        if int(most_active_msg_teamchange):
            es.msg('#multi', '#green[MostActive] #lightgreen%s\'s #defaultplaying times on %s: Total %s - Session %s' % (event_var['es_username'], most_active_kv['teamname'][event_var['oldteam']], _format(_getteamtime(event_var['userid'], event_var['oldteam'])), _format(_getteamtime(event_var['userid'], event_var['oldteam'], True))))

def player_changename(event_var):
    player = _player(event_var['userid'])
    player.name_last = player.player.attributes['name']

    if int(most_active_usertags):
        gamethread.delayed(15, _checkname, (event_var['userid']))

def round_start(event_var):
    for userid in es.getUseridList():
        _calculate(userid)

def round_end(event_var):
    most_active_db.commit()

def saycmd():
    most_active_db.commit()

    if es.getcmduserid() in es.getUseridList():
        arg1 = str(es.getargv(1))
        arg2 = str(es.getargv(2))
        if arg1.lower().startswith('top'):
            if arg2 and arg2 in most_active_kv['teamindex']:
                toplist = most_active_db.cursor()
                toplist.execute('SELECT id,name_last,uniqueid FROM PlayerList ORDER BY time_team%s DESC LIMIT 50;' % int(most_active_kv['teamindex'][arg2]))
                toplist = toplist.fetchall()
                topmenu = popuplib.easymenu('most_active_list_%s' % es.getcmduserid(),None,_player_select)
                topmenu.settitle('Most Active Players\nTeam: %s' % most_active_kv['teamname'][str(most_active_kv['teamindex'][arg2])])
            else:
                toplist = most_active_db.cursor()
                toplist.execute('SELECT id,name_last,uniqueid FROM PlayerList ORDER BY time_total DESC LIMIT 50;')
                toplist = toplist.fetchall()
                topmenu = popuplib.easymenu('most_active_list_%s' % es.getcmduserid(),None,_player_select)
                topmenu.settitle('Most Active Players')
            for player in toplist:
                if not int(most_active_ignore_bots) or not player[2].startswith('BOT'):
                    topmenu.addoption(player[0], player[1])
            if toplist and topmenu:
                topmenu.send(es.getcmduserid())
                most_active_popups['list_%s'%es.getcmduserid()] = topmenu
        elif arg1.lower() == 'help':
            es.tell(es.getcmduserid(), '#multi', '#green[MostActive] #defaultType \'!active [name]\' for personal stats')
            es.tell(es.getcmduserid(), '#multi', '#green[MostActive] #defaultType \'!active top [team (0-4)]\' for top stats')
            if int(most_active_usertags) or int(most_active_usermodels):
                es.tell(es.getcmduserid(), '#multi', '#green[MostActive] #defaultType \'!active status [name]\' for personal reward status')
        elif arg1.lower() == 'status' and (int(most_active_usertags) or int(most_active_usermodels)):
            if len(arg2) and es.getuserid(arg2) > 0:
                _calculate(es.getuserid(arg2))
                _player_status(es.getcmduserid(), _player(es.getuserid(arg2)).id)
            else:
                _calculate(es.getcmduserid())
                _player_status(es.getcmduserid(), _player(es.getcmduserid()).id)
        elif len(arg1) and es.getuserid(arg1) > 0:
            _calculate(es.getuserid(arg1))
            _player_select(es.getcmduserid(), _player(es.getuserid(arg1)).id)
        else:
            _calculate(es.getcmduserid())
            _player_select(es.getcmduserid(), _player(es.getcmduserid()).id)

def _player_select(userid, choice, popupid=None):
    count = most_active_db.cursor()
    count.execute('SELECT COUNT(id) FROM PlayerList;')
    count = count.fetchone()[0]

    player = most_active_db.cursor()
    player.execute('SELECT id,name,name_last,time_total,time_team0,time_team1,time_team2,time_team3 FROM PlayerList WHERE id=? LIMIT 1;', (choice,))
    player = player.fetchone()
    
    if player:
        popup = popuplib.create('most_active_menu_%s' % userid)
        popup.addline('Details for %s' % player[2])
        if player[1] != player[2]:
            popup.addline('    aka %s' % player[1])
        popup.addline('------------------')
        popup.addline('->1. Total Times')
        popup.addline('    Connected: %s' % _format(player[3]))
        popup.addline('    %s: %s' % (most_active_kv['teamname']['0'], _format(player[4])))
        popup.addline('    %s: %s' % (most_active_kv['teamname']['1'], _format(player[5])))
        popup.addline('    %s: %s' % (most_active_kv['teamname']['2'], _format(player[6])))
        popup.addline('    %s: %s' % (most_active_kv['teamname']['3'], _format(player[7])))
        if es.getuserid(player[2]):
            player = _player(es.getuserid(player[2]))
            popup.addline('------------------')
            popup.addline('->2. Session Times')
            popup.addline('    Connected: %s' % _format(player.session_total))
            popup.addline('    %s: %s' % (most_active_kv['teamname']['0'], _format(player.session_team0)))
            popup.addline('    %s: %s' % (most_active_kv['teamname']['1'], _format(player.session_team1)))
            popup.addline('    %s: %s' % (most_active_kv['teamname']['2'], _format(player.session_team2)))
            popup.addline('    %s: %s' % (most_active_kv['teamname']['3'], _format(player.session_team3)))
        popup.addline('------------------')
        popup.addline('->8. Calculated in Realtime')
        popup.addline('->9. Tracking %s Players' % count)
        popup.addline('------------------')
        if popupid and popuplib.exists(popupid):
            popup.addline('->0. Back')
            popup.submenu(10, popuplib.find(popupid))
        else:
            popup.addline('->0. Close')
        popup.send(userid)
        most_active_popups['menu_%s'%userid] = popup

def _player_status(userid, choice):
    count = most_active_db.cursor()
    count.execute('SELECT COUNT(id) FROM PlayerList;')
    count = count.fetchone()[0]

    player = most_active_db.cursor()
    player.execute('SELECT id,name,name_last,time_total FROM PlayerList WHERE id=? LIMIT 1;', (choice,))
    player = player.fetchone()
    
    if player:
        popup = popuplib.create('most_active_menu_%s' % userid)
        popup.addline('Status for %s' % player[2])
        if player[1] != player[2]:
            popup.addline('    aka %s' % player[1])
        popup.addline('------------------')
        popup.addline('->1. Total Times')
        popup.addline('    Connected: %s' % _format(player[3]))
        popup.addline('------------------')
        if int(most_active_usertags):
            nxttime = None
            nxttext = None
            for tagtime in sorted(most_active_kv['usertags'], key=_keyintstr, reverse=True):
                tagtext = most_active_kv['usertags'][str(tagtime)]
                if int(str(tagtime))*60 <= player[3]:
                    popup.addline('    Current: %s' % tagtext.replace('%', ''))
                    if nxttime and nxttext:
                        popup.addline('    Next: %s' % nxttext.replace('%', ''))
                        popup.addline('    Left: %s' % _format(int(str(nxttime))*60-player[3]))
                    else:
                        popup.addline('    Next: n/a')
                    popup.addline('------------------')
                    break
                nxttime = tagtime
                nxttext = tagtext
            if nxttime and nxttext and nxttime == tagtime:
                popup.addline('    Next: %s' % nxttext.replace('%', ''))
                popup.addline('    Left: %s' % _format(int(str(nxttime))*60-player[3]))
                popup.addline('------------------')
        if int(most_active_usermodels):
            nxttime = None
            nxttext = None
            for modeltime in sorted(most_active_kv['usertags'], key=_keyintstr, reverse=True):
                modeltext = most_active_kv['usertags'][str(modeltime)]
                if int(str(modeltime))*60 <= player[3]:
                    popup.addline('    Current: %s' % os.path.basename(modeltext))
                    if nxttime and nxttext:
                        popup.addline('    Next: %s' % os.path.basename(nxttext))
                        popup.addline('    Left: %s' % _format(int(str(nxttime))*60-player[3]))
                    else:
                        popup.addline('    Next: n/a')
                    popup.addline('------------------')
                    break
                nxttime = modeltime
                nxttext = modeltext
            if nxttime and nxttext and nxttime == tagtime:
                popup.addline('    Next: %s' % os.path.basename(nxttext))
                popup.addline('    Left: %s' % _format(int(str(nxttime))*60-player[3]))
                popup.addline('------------------')
        popup.addline('->8. Calculated in Realtime')
        popup.addline('->9. Tracking %s Players' % count)
        popup.addline('------------------')
        popup.addline('->0. Close')
        popup.send(userid)
        most_active_popups['menu_%s'%userid] = popup

def _player(userid):
    if not int(userid) in most_active_players:
        most_active_players[int(userid)] = Player(userid)
    try:
        most_active_players[int(userid)].player.refreshAttributes()
    except:
        pass
    return most_active_players[int(userid)]

def _calculate(userid, teamid=None):
    player = _player(userid)
    teamid = int(teamid) if not teamid is None else int(player.player.attributes['teamid'])
    if teamid == 0:
        player.time_team0 += int(_gettimestamp())-player.time_last
    elif teamid == 1:
        player.time_team1 += int(_gettimestamp())-player.time_last
    elif teamid == 2:
        player.time_team2 += int(_gettimestamp())-player.time_last
    elif teamid == 3:
        player.time_team3 += int(_gettimestamp())-player.time_last
    player.time_total += int(_gettimestamp())-player.time_last
    player.time_last = int(_gettimestamp())

def _gettotaltime(userid, session=False):
    player = _player(userid)
    if not session:
        return player.time_total
    else:
        return player.session_total

def _getteamtime(userid, teamid=None, session=False):
    player = _player(userid)
    teamid = int(teamid) if not teamid is None else int(player.player.attributes['teamid'])
    if not session:
        if teamid == 0:
            return player.time_team0
        elif teamid == 1:
            return player.time_team1
        elif teamid == 2:
            return player.time_team2
        elif teamid == 3:
            return player.time_team3
    else:
        if teamid == 0:
            return player.session_team0
        elif teamid == 1:
            return player.session_team1
        elif teamid == 2:
            return player.session_team2
        elif teamid == 3:
            return player.session_team3

def _checkname(userid, join=False):
    if int(userid) in es.getUseridList():
        player = _player(userid)
        if int(player.player.attributes['isbot']) == 0:
            newname = player.player.attributes['name']
            for tagtime in most_active_kv['usertags']:
                tagtext = most_active_kv['usertags'][str(tagtime)]
                if int(str(tagtime))*60 > player.time_total and tagtext.replace('%') in player.player.attributes['name'] and not player.player.attributes['steamid'] in most_active_kv['usertags_idlist'] and not int(most_active_kv['usertags_idlist'][player.player.attributes['steamid']]) == 2:
                    newname = newname.replace(tagtext.replace('%'), '')
                    es.cexec(userid, 'name %s' % player.player.attributes['name'].replace(tagtext.replace('%'), ''))
                    es.tell(userid, '#multi', '#green[MostActive] #lightgreenYou are using a protected nickname tag! Remove it or you will stay blinded!')
                    player.player.set('flash', (255, 5))
                    gamethread.delayed(5, _checkname, (userid))
            if join:
                for tagtime in sorted(most_active_kv['usertags'], key=_keyintstr, reverse=True):
                    tagtext = most_active_kv['usertags'][str(tagtime)]
                    if int(str(tagtime))*60 <= player.time_total and not tagtext.replace('%') in player.player.attributes['name'] and not player.player.attributes['steamid'] in most_active_kv['usertags_idlist'] and not int(most_active_kv['usertags_idlist'][player.player.attributes['steamid']]) == 1:
                        es.cexec(userid, 'name %s' % tagtext.replace('%', newname))
                        es.tell(userid, '#multi', '#green[MostActive] #lightgreenYou are allowed to use a protected nickname tag: %s' % newname)
                        break

def _checkmodel(userid):
    if int(userid) in es.getUseridList():
        player = _player(userid)
        if int(player.player.attributes['isbot']) == 0:
            for modeltime in sorted(most_active_kv['usermodels'], key=_keyintstr, reverse=True):
                modeltext = most_active_kv['usermodels'][str(modeltime)]
                if int(str(modeltime))*60 <= player.time_total and not player.player.attributes['steamid'] in most_active_kv['usermodels_idlist'] and not int(most_active_kv['usermodels_idlist'][player.player.attributes['steamid']]) == 1:
                    player.player.set('model', modeltext)
                    break

def _keyintstr(key):
    return int(str(key))

def _gettimestamp():
    return time.time()-1136049334 # es.gettimestamp emulation

def _format(duration):
    _format = ''
    if not duration or not int(duration):
        return 'n/a'
    else:
        duration = int(duration)
    if duration >= 24*60*60:
        _format += '%sd ' % int(duration/24*60*60)
        duration %= 24*60*60
    if duration >= 60*60:
        _format += '%sh ' % int(duration/60*60)
        duration %= 60*60
    if duration >= 60:
        _format += '%sm ' % int(duration/60)
        duration %= 60
    if duration:
        _format += '%ss ' % int(duration)
    if _format.rstrip():
        return _format.rstrip()
